;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Software License Agreement (BSD License)
;; 
;; Copyright (c) 2008, Willow Garage, Inc.
;; All rights reserved.
;;
;; Redistribution and use in source and binary forms, with 
;; or without modification, are permitted provided that the 
;; following conditions are met:
;;
;;  * Redistributions of source code must retain the above 
;;    copyright notice, this list of conditions and the 
;;    following disclaimer.
;;  * Redistributions in binary form must reproduce the 
;;    above copyright notice, this list of conditions and 
;;    the following disclaimer in the documentation and/or 
;;    other materials provided with the distribution.
;;  * Neither the name of Willow Garage, Inc. nor the names 
;;    of its contributors may be used to endorse or promote 
;;    products derived from this software without specific 
;;    prior written permission.
;; 
;; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND 
;; CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED 
;; WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
;; WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
;; PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
;; COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
;; INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
;; CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
;; PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;; DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
;; CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
;; CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
;; OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
;; SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
;; DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package sb-rt)

;;; copy of two functions from sb-rt, extended to return more values instead of just test-symbol or nil, and to measure time. Else code remained unchanged

(defun do-entry (entry &optional
                         (s *standard-output*))
  (let ((report nil)
        (start-time (get-internal-real-time )))
    (catch '*in-test*
      (setq *test* (name entry))
      (setf (pend entry) t)
      (let* ((*in-test* t)
             ;; (*break-on-warnings* t)
             (aborted nil)
             r)
        ;; (declare (special *break-on-warnings*))

        (block aborted
          (setf r
                (flet ((%do
                         ()
                         (if *compile-tests*
                           (multiple-value-list
                            (funcall (compile
                                      nil
                                      `(lambda ()
                                         (declare
                                          (optimize ,@*optimization-settings*))
                                         ,(form entry)))))
                           (multiple-value-list
                            (eval (form entry))))))
                  (if *catch-errors*
                    (handler-bind
                        ((style-warning #'muffle-warning)
                         (error #'(lambda (c)
                                    (setf aborted t)
                                    (setf r (list c))
                                    (return-from aborted nil))))
                      (%do))
                    (%do)))))

        (setf (pend entry)
              (or aborted
                  (not (equalp-with-case r (vals entry)))))

       
        (let ((*print-circle* *print-circle-on-failure*)
              (sum-time (- (get-internal-real-time) start-time)))
          (if (pend entry)
            (progn
              (format s "~&Test ~:@(~S~) failed~
                   ~%Form: ~S~
                   ~%Expected value~P: ~
                      ~{~S~^~%~17t~}~%"
                      *test* (form entry)
                      (length (vals entry))
                      (vals entry))
              (format s "Actual value  ~P: ~
                      ~{~S~^~%~15t~}.~%"
                      (length r) r)
              (setf report (list *test* sum-time (form entry) (vals entry) r)))
            (setf report (list *test* sum-time))))))
    (values (when (not (pend entry)) *test*) report)))


(defun do-entries (s)
  (format s "~&Doing ~A pending test~:P ~
             of ~A tests total.~%"
          (count t (cdr *entries*)
                 :key #'pend)
          (length (cdr *entries*)))
  (let ((result)
        (start-time (get-internal-real-time )))
    (dolist (entry (cdr *entries*))
      (when (pend entry)
        (multiple-value-bind (test report) (do-entry entry s)
          (push report result)
          ;; print unless nil?
          (format s "~@[~<~%~:; ~:@(~S~)~>~]" test))))
    (let ((pending (pending-tests))
          (expected-table (make-hash-table :test #'equal)))
      (dolist (ex *expected-failures*)
        (setf (gethash ex expected-table) t))
      (let ((new-failures
              (loop for pend in pending
                    unless (gethash pend expected-table)
                      collect pend)))
        (if (null pending)
          (format s "~&No tests failed.")
          (progn
            (format s "~&~A out of ~A ~
                   total tests failed: ~
                   ~:@(~{~<~%   ~1:;~S~>~
                         ~^, ~}~)."
                    (length pending)
                    (length (cdr *entries*))
                    pending)
            (if (null new-failures)
              (format s "~&No unexpected failures.")
              (when *expected-failures*
                (format s "~&~A unexpected failures: ~
                   ~:@(~{~<~%   ~1:;~S~>~
                         ~^, ~}~)."
                        (length new-failures)
                        new-failures)))))
        (finish-output s)
        (values (null pending) result (- (get-internal-real-time) start-time))))))





