;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Software License Agreement (BSD License)
;; 
;; Copyright (c) 2008, Willow Garage, Inc.
;; All rights reserved.
;;
;; Redistribution and use in source and binary forms, with 
;; or without modification, are permitted provided that the 
;; following conditions are met:
;;
;;  * Redistributions of source code must retain the above 
;;    copyright notice, this list of conditions and the 
;;    following disclaimer.
;;  * Redistributions in binary form must reproduce the 
;;    above copyright notice, this list of conditions and 
;;    the following disclaimer in the documentation and/or 
;;    other materials provided with the distribution.
;;  * Neither the name of Willow Garage, Inc. nor the names 
;;    of its contributors may be used to endorse or promote 
;;    products derived from this software without specific 
;;    prior written permission.
;; 
;; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND 
;; CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED 
;; WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
;; WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
;; PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
;; COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
;; INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
;; CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
;; PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
;; DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
;; CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
;; CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
;; OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
;; SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
;; DAMAGE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :gtest-adapter)

;; wraps calls to rt to provide data structures compatible with gtest

(defun create-rt-do-tests-fun (name)
  "Creates a gtest compatible function which wraps sb-rt:do-tests to capture values and return a list of name, results and time"
  (lambda (arg-struct)
    (declare (ignore arg-struct))
  ;; the output of patched rt is (values pending result time)
    (multiple-value-bind (pending result time) (do-tests)
      (declare (ignore pending))
      ;; insert name here instead of result
      (list name result time))))


(defun create-rt-do-suite-fun (name)
  "Creates a gtest compatible function which wraps sb-rt:do-suite to capture values and return a list of name, results and time"
  (lambda (arg-struct)
    (declare (ignore arg-struct))
    ;; the output of patched rt is (values pending result time)
    (multiple-value-bind (pending result time) (do-suite name)
      (declare (ignore pending))
      ;; insert name here instead of result
      (list name result time))))



(defun rt->gtest (result-list sysout syserr)
  "Takes output of patched sb-rt:do-tests and creates struct. Results must be list (test-case-name testcase-time failure-form? expected-result? actual-result?)."
  (destructuring-bind
      (suite-name
       results
       suite-time)
      result-list
    (make-gtestsuite
     :name suite-name
     :time suite-time
     :sysout sysout
     :syserr syserr
     :testcases
     (loop for result in results
           collect
           (make-gtestcase
            :classname (first result)   ; could also make nil
            :name (first result)
            :time (second result)
            :failures (unless (null (some #'identity (cddr result)))
                        (list
                         (make-gtestfailure
                          :type "AssertionFailure"
                          :message (format nil "Form ~a expected ~a but got ~a"
                                           (third result)
                                           (fourth result)
                                           (fifth result))))))))))

(defun rt-do-tests->gtest (name posix-args)
  "Calls rt do-tests and transforms results for gtest printing"
  (run-suites-write-gtest-file
   posix-args
   (list (create-rt-do-tests-fun name))
   'rt->gtest))


(defun rt-do-suites->gtest (name-list posix-args)
  "Calls rt do-tests and transforms results for gtest printing"
  (run-suites-write-gtest-file
   posix-args
   (mapcar (lambda (name) (create-rt-do-suite-fun name)) name-list)
   'rt->gtest))