(in-package :gtest-adapter)


(defparameter *rt-test-result* nil)
(defparameter *suite-test-result* nil)
(defparameter *count-test-runs* 0)

(labels
  ;; can't use rt to test rt directly because of context sensitivity
  ;; of rem-all-tests and deftest. As a hack, we test it using asserts
  ;; at load time, store the result in a var, and use an rt deftest to
  ;; check the var. The closure just makes sure IDEs rerun the tet
  ;; when recompiling the function.
  ((test-rt->gtest-output ()
    "Asserts-based test of xml generation based on sucessful and failed tests."
    (rem-all-tests) ;; only test these tests
  
    (deftest test-addition
        (+ 21 22)
      42)

    (deftest test-substraction
        (- 21 22)
      -1)

    (deftest test-division
        (/ 2 0)
      4)

    (deftest test-sleep
        (sleep 1)
      nil)

    "binary main entry function.."
    ;; TODO: write this as a node, use launch file to start adapto roslaunch file
    (let ((result
            (with-output-to-string (stream)
              (run-suite-write-to-stream stream (create-rt-do-tests-fun "suite") nil #'rt->gtest :no-output t))))
      (rem-all-tests) ;; loading this file should not define tests above as tests
      ;; asserting bits only to be flexible
      (assert (search "<testsuite name=\"suite\" tests=\"2\" failures=\"2\" errors=\"0\"" result))
      (assert (search "<testcase classname=\"TEST-SLEEP\" name=\"TEST-SLEEP\" status=\"run\"" result))
      (assert (search "<failure type=\"AssertionFailure\">Form (/ 2 0) expected (4) but got (arithmetic error DIVISION-BY-ZERO signalled" result))
      (assert (search "<failure type=\"AssertionFailure\">Form (+ 21 22) expected (42) but got (43)</failure>" result))
      (assert (search "</testsuite>" result)))
    :ok)

   (test-suite-functions ()
     (let ((suitename (gensym))
           (entries))
       (sb-rt::rem-all-suites)
       (assert (not (null sb-rt::*rt-suites*)))
       (assert (= 0 (hash-table-count sb-rt::*rt-suites*)))
       (deftest fail1
           t
         nil)
       (rem-all-tests)
       (deftest test1
           (progn
             (incf *count-test-runs*)
             t)
         t)
       (setf entries sb-rt::*entries*)
       (create-suite suitename)
       (rem-all-tests)
       (deftest fail1
           t
         nil)
       ;; make sure test1 is only one captured in suite
       (assert (= 1 (hash-table-count sb-rt::*rt-suites*)))
       (assert (equal
                (gethash suitename sb-rt::*rt-suites*)
                entries))
       (rem-all-tests)
       (assert (= 0 *count-test-runs*))
       (sb-rt::do-suites)
       (sb-rt::do-suite suitename)
       (assert (= 2 *count-test-runs*))
       
       :ok)))
  
  ;; this is a trick to get the result from rt using rt
  (setf *rt-test-result*
    (handler-case (test-rt->gtest-output)
      (error (e) (format nil "~a" e))))

  (setf *suite-test-result*
    (handler-case (test-suite-functions)
      (error (e) (format nil "~a" e))))); end closure


;; Now the rt based testing begins, so removing all artifacts
(rem-all-tests)


(deftest test-rt-wrapping
    *rt-test-result*
  :ok)

(deftest test-suites
    *suite-test-result*
  :ok)

(deftest test-serializing-nils
    (with-output-to-string (stream)
      (serialize-testsuite stream
                           (make-gtestsuite :name nil :time nil
                                            :testcases nil
                                            :sysout nil :syserr nil)))
"<testsuite name=\"NIL\" tests=\"0\" failures=\"0\" errors=\"0\" time=\"0.0\">
<system-out><![CDATA[NIL]]></system-out>
<system-err><![CDATA[NIL]]></system-err>
</testsuite>
")

(deftest test-escaping
    (with-output-to-string (stream)
      (serialize-testsuite stream (make-gtestsuite :name "\"foo" :time "3" :testcases nil :sysout "\"foo]]>bar" :syserr "\"foo]]>bar")))
  "<testsuite name=\"&quot;foo\" tests=\"0\" failures=\"0\" errors=\"0\" time=\"3.0\">
<system-out><![CDATA[\"foo]]]]><![CDATA[]bar]]></system-out>
<system-err><![CDATA[\"foo]]]]><![CDATA[]bar]]></system-err>
</testsuite>
")

(deftest test-serializing-testcases-nils
    (with-output-to-string (stream)
      (serialize-testsuite
       stream
       (make-gtestsuite
        :name nil :time nil :sysout nil :syserr nil
        :testcases (list
                    (make-gtestcase :classname nil :name nil :time nil :failures nil)))))
  "<testsuite name=\"NIL\" tests=\"1\" failures=\"0\" errors=\"0\" time=\"0.0\">
<testcase classname=\"NIL\" name=\"NIL\" status=\"run\" time=\"0.0\">
</testcase>
<system-out><![CDATA[NIL]]></system-out>
<system-err><![CDATA[NIL]]></system-err>
</testsuite>
")

(deftest test-escaping-testcases
    (with-output-to-string (stream)
      (serialize-testsuite
       stream
       (make-gtestsuite
        :name nil :time nil :sysout nil :syserr nil
        :testcases (list
                    (make-gtestcase :classname "\"foo" :name "\"bar" :time 7 :failures nil)))))
  "<testsuite name=\"NIL\" tests=\"1\" failures=\"0\" errors=\"0\" time=\"0.0\">
<testcase classname=\"&quot;foo\" name=\"&quot;bar\" status=\"run\" time=\"7.0\">
</testcase>
<system-out><![CDATA[NIL]]></system-out>
<system-err><![CDATA[NIL]]></system-err>
</testsuite>
")



(deftest test-serializing-failures-nils
    (with-output-to-string (stream)
      (serialize-testsuite
       stream
       (make-gtestsuite
        :name nil :time nil :sysout nil :syserr nil
        :testcases (list
                    (make-gtestcase
                     :classname nil :name nil :time nil
                     :failures (list
                                (make-gtestfailure :type nil :message nil)))))))
  "<testsuite name=\"NIL\" tests=\"0\" failures=\"1\" errors=\"0\" time=\"0.0\">
<testcase classname=\"NIL\" name=\"NIL\" status=\"run\" time=\"0.0\">
<failure type=\"NIL\">NIL</failure>
</testcase>
<system-out><![CDATA[NIL]]></system-out>
<system-err><![CDATA[NIL]]></system-err>
</testsuite>
")


(deftest test-escaping-failures
    (with-output-to-string (stream)
      (serialize-testsuite
       stream
       (make-gtestsuite
        :name nil :time nil :sysout nil :syserr nil
        :testcases (list
                    (make-gtestcase
                     :classname nil :name nil :time nil
                     :failures (list
                                (make-gtestfailure :type "\"foo" :message "\"bar<foo>tee'bla&and")))))))
  "<testsuite name=\"NIL\" tests=\"0\" failures=\"1\" errors=\"0\" time=\"0.0\">
<testcase classname=\"NIL\" name=\"NIL\" status=\"run\" time=\"0.0\">
<failure type=\"&quot;foo\">&quot;bar&lt;foo&gt;tee&#039;bla&amp;and</failure>
</testcase>
<system-out><![CDATA[NIL]]></system-out>
<system-err><![CDATA[NIL]]></system-err>
</testsuite>
")

(deftest test-parse-arg
    (gtestargs-output-xml (get-gtest-args '("--gtest=nothisone" "--gtest_output=xml:/foo/bar" "--gtest=nothisone")))
  "/foo/bar")

;; used for rostest based self test
(defun roslisp-testing-selftest-main ()
  "runs the above tests and writes results to xml as gtest would"
  (rt-do-tests->gtest "roslisp-testing-selftest" sb-ext:*posix-argv*))

;; acting in the package under test, adding export of main function for executable
(export 'roslisp-testing-selftest-main )