(in-package :test-roslisp)

(rem-all-tests)

(def-fixture with-clean-roslisp-vars ()
  (let* ((roslisp::*namespace* "/")
         (roslisp::*ros-log-location* nil)
         (result (&body)))
    result))

(deftest parse-remapping
    (roslisp::parse-remapping "foo:=bar")
  "foo" "bar")

(deftest parse-remapping-nil
    (roslisp::parse-remapping "foo=bar")
  nil)

(deftest handle-command-line-args-nodename
    (with-fixture with-clean-roslisp-vars nil
        (roslisp::handle-command-line-arguments "foonode" nil)
           (list
            roslisp::*namespace*
            roslisp::*ros-node-name*
            (hash-table-count roslisp::*remapped-names*)
            roslisp::*ros-log-location*))
  ("/" "/foonode" 0 nil))

;; (deftest handle-command-line-args-namespace
;;     (labels ((roslisp::postprocess-namespace (x) "fooname"))
;;       (roslisp::handle-command-line-arguments "foonode" nil)
;;       (list
;;        roslisp::*namespace*
;;        roslisp::*ros-node-name*
;;        (hash-table-count roslisp::*remapped-names*)))
;;   ("/fooname" "/fooname/foonode" 0))

(deftest handle-command-line-args-mappings
    (with-fixture with-clean-roslisp-vars nil
      (roslisp::handle-command-line-arguments "foonode" "foo:=bar baz:=bin")
      (list
       roslisp::*namespace*
       roslisp::*ros-node-name*
       roslisp::*ros-log-location*
       (hash-table-count roslisp::*remapped-names*)
       (gethash "/foo" roslisp::*remapped-names*)
       (gethash "/baz" roslisp::*remapped-names*)))
  ("/" "/foonode" nil 2 "/bar" "/bin"))

(deftest handle-command-line-args-remap-namespace-last
    (with-fixture with-clean-roslisp-vars nil
      (roslisp::handle-command-line-arguments "foonode" "foo:=bar baz:=bin __ns:=fooname")
      (list
       roslisp::*namespace*
       roslisp::*ros-node-name*
       roslisp::*ros-log-location*
       (hash-table-count roslisp::*remapped-names*)
       (gethash "/foo" roslisp::*remapped-names*)
       (gethash "/baz" roslisp::*remapped-names*)))
  ("/fooname/" "/fooname/foonode" nil 2 "/bar" "/bin"))

(deftest handle-command-line-args-remap-namespace-first
    (with-fixture with-clean-roslisp-vars nil
      (roslisp::handle-command-line-arguments "foonode" "__ns:=fooname foo:=bar baz:=bin")
      (list
       roslisp::*namespace*
       roslisp::*ros-node-name*
       roslisp::*ros-log-location*
       (hash-table-count roslisp::*remapped-names*)
       (gethash "/fooname/foo" roslisp::*remapped-names*)
       (gethash "/fooname/baz" roslisp::*remapped-names*)))
  ("/fooname/" "/fooname/foonode" nil 2 "/fooname/bar" "/fooname/bin"))

(deftest handle-command-line-args-remap-nodename
    (with-fixture with-clean-roslisp-vars nil (roslisp::handle-command-line-arguments "foonode" "foo:=bar baz:=bin __name:=fooname")
      (list
       roslisp::*namespace*
       roslisp::*ros-node-name*
       roslisp::*ros-log-location*
       (hash-table-count roslisp::*remapped-names*)
       (gethash "/foo" roslisp::*remapped-names*)
       (gethash "/baz" roslisp::*remapped-names*)))
  ("/" "/fooname" nil 2 "/bar" "/bin"))

(deftest handle-command-line-args-remap-logloc
    (with-fixture with-clean-roslisp-vars nil
      (roslisp::handle-command-line-arguments "foonode" "foo:=bar baz:=bin __log:=foologloc")
      (list
       roslisp::*namespace*
       roslisp::*ros-node-name*
       roslisp::*ros-log-location*
       (hash-table-count roslisp::*remapped-names*)
       (gethash "/foo" roslisp::*remapped-names*)
       (gethash "/baz" roslisp::*remapped-names*)))
  ("/" "/foonode" "foologloc" 2 "/bar" "/bin"))

(deftest handle-command-line-args-remap-params
    (with-fixture with-clean-roslisp-vars nil
      (list
       (roslisp::handle-command-line-arguments "foonode" "_foo:=bar")
       roslisp::*namespace*
       roslisp::*ros-node-name*
       roslisp::*ros-log-location*
       (hash-table-count roslisp::*remapped-names*)))
  ((("~foo" . "bar")) "/" "/foonode" nil 0))


(create-suite "roslisp-command-line-args-test")