(in-package :test-roslisp)

(rem-all-tests)

(def-fixture with-clean-roslisp-vars ()
  (let* ((roslisp::*namespace* "/ns/")
         (roslisp::*ros-node-name* "/foonode")
         (result (&body)))
    result))

(deftest concatenate-ros-names
    (roslisp::concatenate-ros-names '("foo" "bar" "baz"))
    "foo/bar/baz")

(deftest concatenate-ros-names-single
    (roslisp::concatenate-ros-names '("foo"))
    "foo")

(deftest compute-global-name
    (roslisp::compute-global-name nil nil "/foo" )
  "/foo")

(deftest compute-global-name-node
    (roslisp::compute-global-name nil "/bar" "~foo" )
  "/bar/foo")

(deftest compute-global-name-ns
    (roslisp::compute-global-name "/fooname/" nil "foo" )
  "/fooname/foo")

(deftest fully-qualified-name
    (with-fixture with-clean-roslisp-vars nil
      (roslisp:fully-qualified-name "/foo"))
  "/foo")

(deftest fully-qualified-name-node
    (with-fixture with-clean-roslisp-vars nil
      (let ((roslisp::*namespace* "/ns/")
            (roslisp::*ros-node-name* "/foonode"))
        (roslisp:fully-qualified-name "~foo")))
  "/foonode/foo")

(deftest fully-qualified-name-ns
    (with-fixture with-clean-roslisp-vars nil
      (let ((roslisp::*namespace* "/ns/")
          (roslisp::*ros-node-name* "/foonode"))
        (roslisp:fully-qualified-name "foo")))
  "/ns/foo")

(deftest with-fully-qualified-name
    (with-fixture with-clean-roslisp-vars nil
      (let (name
            (orig "/foo"))
        (roslisp::with-fully-qualified-name orig
          (setf name orig))))
  "/foo")

(deftest with-fully-qualified-name-node
    (with-fixture with-clean-roslisp-vars nil
      (let (name
            (roslisp::*namespace* "/ns/")
            (roslisp::*ros-node-name* "/foonode")
            (orig "~foo"))
        (roslisp::with-fully-qualified-name orig
          (setf name orig))))
  "/foonode/foo")

(deftest with-fully-qualified-name-ns
    (with-fixture with-clean-roslisp-vars nil
      ;; fixture code broken, does not work in executable
      (let (name
            (orig "foo")
            (roslisp::*namespace* "/ns/")
            (roslisp::*ros-node-name* "/foonode"))
        (roslisp::with-fully-qualified-name orig
          (setf name orig))))
  "/ns/foo")

(create-suite "roslisp-namespace-test")