(in-package :test-roslisp)

(rem-all-tests)

(deftest make-queue
    (let ((queue (roslisp-queue:make-queue)))
      (list
       (roslisp-queue::head queue)
       (roslisp-queue::tail queue)
       (roslisp-queue::max-size queue)
       (roslisp-queue:queue-size queue)
       (roslisp-queue:peek-front queue)
       (roslisp-queue:queue-empty queue)
       (roslisp-queue:dequeue queue)))
  (nil nil roslisp-extended-reals:infty  0 nil t nil))


(deftest make-queue-init-and-max
    (let ((queue (roslisp-queue:make-queue :sequence '(1 2 3) :max-size 7)))
      (list
       (roslisp-queue::head queue)
       (roslisp-queue::tail queue)
       (roslisp-queue::max-size queue)
       (roslisp-queue:queue-size queue)
       (roslisp-queue:peek-front queue)
       (roslisp-queue:queue-empty queue)))
  ((1 2 3) (3) 7 3 1 nil))

(deftest make-queue-init-array
    (let ((queue (roslisp-queue:make-queue :sequence (make-array '(3) :initial-contents '(1 2 3)) :max-size 7)))
      (list
       (roslisp-queue::head queue)
       (roslisp-queue::tail queue)
       (roslisp-queue::max-size queue)
       (roslisp-queue:queue-size queue)
       (roslisp-queue:peek-front queue)
       (roslisp-queue:queue-empty queue)))
  ((1 2 3) (3) 7 3 1 nil))

;; pending roslisp bugfix release
;; (deftest make-queue-init-greater-than-max
;;     (type-of
;;      (with-fixture roslisp-testing:error-caught ()
;;        (roslisp-queue:make-queue :sequence '(1 2 3) :max-size 2)))
;;   simple-error)

(deftest enqueue
    (let ((queue (roslisp-queue:make-queue :sequence '(1 2) :max-size 4)))
      (roslisp-queue:enqueue 5 queue)
      (list
       (roslisp-queue::head queue)
       (roslisp-queue::tail queue)
       (roslisp-queue::max-size queue)
       (roslisp-queue:queue-size queue)
       (roslisp-queue:peek-front queue)
       (roslisp-queue:queue-empty queue)))
  ((1 2 5) (5) 4 3 1 NIL))


(deftest enqueue-over-maxsize
    (let ((queue (roslisp-queue:make-queue :sequence '(1 2) :max-size 4)))
      (roslisp-queue:enqueue 5 queue)
      (roslisp-queue:enqueue 6 queue)
      (roslisp-queue:enqueue 7 queue)
      (roslisp-queue:enqueue 8 queue)
      (roslisp-queue:enqueue 9 queue)
      (roslisp-queue:enqueue 10 queue)
      (list
       (roslisp-queue::head queue)
       (roslisp-queue::tail queue)
       (roslisp-queue::max-size queue)
       (roslisp-queue:queue-size queue)
       (roslisp-queue:peek-front queue)
       (roslisp-queue:queue-empty queue)))
  ((7 8 9 10) (10) 4 4 7 NIL))

;; pending roslisp bugfix release
;; (deftest enqueue-dequeue
;;     (let ((queue (roslisp-queue:make-queue :sequence '(1 2) :max-size 4)))
;;       (roslisp-queue:enqueue 5 queue)
;;       (roslisp-queue:enqueue 6 queue)
;;       (roslisp-queue:enqueue 7 queue)
;;       (list
;;        (roslisp-queue:peek-front queue)
;;        (roslisp-queue:dequeue queue)
;;        (roslisp-queue:peek-front queue)
;;        (roslisp-queue:dequeue queue)
;;        (roslisp-queue:peek-front queue)
;;        (roslisp-queue:dequeue queue)
;;        (roslisp-queue:peek-front queue)
;;        (roslisp-queue:dequeue queue)
;;        (roslisp-queue:peek-front queue)
;;        (roslisp-queue:dequeue queue)))
;;   (2 2 5 5 6 6 7 7 nil nil ))


(create-suite "roslisp-queue-test")