(in-package :test-roslisp)

;;;; service provider used below 


(def-service-callback (add test_roslisp-srv:test) (m)
  (with-fields ((a (data a)) (b (data b))) m
    (when (= a b 0)
      (error "test-error"))
    (when (= a b 1)
      (error 'service-error :format-control "test-error"))
    (make-response :sum (+ a b))))

(defun service-server ()
  (with-ros-node ("test_server" :spin t)
    (register-service-fn "adder" #'add 'test_roslisp-srv:test)))


;;; tests against server above

(rem-all-tests)

(deftest test-send-receive
    (with-ros-node ("test_client")
      (when (wait-for-service "adder" 20)
        (sum (call-service "adder" 'Test
                           :m (make-msg "test_roslisp/TwoInts" ;; 'test_roslisp-msg:twoints
                                        
                                        (data a) 24 (data b) 42)))))
    66)

(deftest test-send-receive-error-message
    ;; calls service with values causing service error with specific error msg
    (with-ros-node ("test_client")
      (handler-case
          (progn
            (wait-for-service "adder" 20)
            (call-service "adder" 'Test
                          :m (make-msg "test_roslisp/TwoInts";; 'test_roslisp-msg:twoints
                                       ;; 1 and 1 should trigger service-error with message test-error
                                       (data a) 1 (data b) 1))
            (ros-error (service-client-test) "Expected error")
            "Missing expected error")
        (error (e)
          ;; check error message got through
          (if (search "test-error" (format nil "~a" e))
            t
            (format nil "~a" e)))))
  t)


(deftest test-send-receive-error
    ;; calls service with values that cause error in service callback
    (with-ros-node ("test_client")
      (handler-case
          (progn
            (wait-for-service "adder" 20)
            (call-service "adder" 'Test
                          :m (make-msg "test_roslisp/TwoInts";;'test_roslisp-msg:twoints
                                       ;; 0 and 0 should trigger error
                                       (data a) 0 (data b) 0))
            (ros-error (service-client-test) "Expected error")
            "Missing expected error")
        (error () t)))
  t)


(create-suite "roslisp-service-test")
