(in-package :test-roslisp)

;;; the talker and listener functions in this file define nodes that will be tested against in topic-tests.lisp


;;;;; Nodes using std_msgs/String
(defun talker ()
  "Periodically print a string message on the /chatter topic"
  (with-ros-node ("talker")
    (let ((pub (advertise "chatter" "std_msgs/String" :latch nil))
	  (i 0))
      (loop-at-most-every .1
	 (publish pub (make-instance 'std_msgs-msg:string :data (format nil "foo ~a" (incf i))))))))


(defun listener ()
  "publishes on chatter_echo what it receives on chatter, inversed"
  (with-ros-node ("listener" :spin t :anonymous t)
    (advertise "chatter_echo" "std_msgs/String" :latch t)
    (sleep 3)
    (subscribe "chatter" "std_msgs/String" #'(lambda (m) (with-fields (data) m (publish "chatter_echo" (make-instance 'std_msgs-msg:string :data (reverse data))))))))


(defun broken-listener ()
  "subscribes to error_chatter topic raising error in the callback when
  \"error\" is sent, to test roslisp does not go down on errors in
  callbacks. Else sends back message string reversed on error_chatter_echo"
  (with-ros-node ("broken_listener" :spin t :anonymous t)
    (advertise "error_chatter_echo" "std_msgs/String" :latch t)
    (sleep 3)
    (subscribe "error_chatter" "std_msgs/String"
               #'(lambda (m)
                   (with-fields (data) m
                     (when (string= "error" data)
                       (error "test-error"))
                     (publish "error_chatter_echo" (make-instance 'std_msgs-msg:string :data (reverse data))))))))


;;;;; Nodes using test_roslisp/ComplexMessage

(defun complex-listener ()
  (with-ros-node ("complex_listener" :spin t :anonymous t)
    (advertise "complex_chatter_echo" "test_roslisp/ComplexMessage")
    (sleep 3)
    (subscribe "complex_chatter" "test_roslisp/ComplexMessage"
               #'(lambda (m)
                   (publish "complex_chatter_echo" m)))))


(defun complex-talker ()
  (with-ros-node ("complex_talker")
    (let ((ping-pub (advertise "ping" "test_roslisp/ComplexMessage"))
	  (pang-pub (advertise "pang" "test_roslisp/ComplexMessage")))
      (subscribe "pong" "test_roslisp/ComplexMessage" 
		 #'(lambda (m) (publish pang-pub m)))
      (loop-at-most-every .1
	 (publish ping-pub (make-complex-message))))))

;; makes sure each message gets unique id
(defvar *c* 0)


(defun make-complex-message ()
  (incf *c*)
  (make-instance 'ComplexMessage
		 :x #(1.1 2.2 3.3 4.4)
		 :y (make-foo-array 3)
		 :z (make-foo-array 2)
		 :ind (mod *c* (* 2 4))
		 :w (vector (make-instance 'std_msgs-msg:String :data "foo")
			    (make-instance 'std_msgs-msg:String :data "bar"))
		 :qux (evenp *c*)))


(defun make-foo-array (n)
  (let ((a (make-array n)))
    (dotimes (i n a)
      (setf (aref a i)
	    (make-instance 'Foo
			   :y (make-bar 1)
			   :z (vector (make-bar 2) (make-bar 3)))))))

(defun make-bar (i)
  (make-instance 'Bar :y i))