(in-package :test-roslisp)

(rem-all-tests)

(defun equal-message (msg1 msg2)
  "tests equalness of messages by comparing print output"
  (equal (format nil "~a" msg1) (format nil "~a" msg2)))

(deftest test-send-receive-complex
    ;; depends on remote node being started, talking on ping, and echoing on pang what it receives on pong
    (let ((ok-flag t)
          (num-received 0)
          (buffer (make-array '(8))))
      (with-ros-node ("roslisp-tester")
        (let ((pub (advertise "pong" 'test_roslisp-msg:ComplexMessage)))
          (labels ((ping-callback (m)
                     (with-fields (ind) m
                       (setf (aref buffer ind) m)
                       (publish pub m)))
                   (pang-callback (m)
                     (with-fields (ind) m
                       (setf ok-flag (and ok-flag (equal-message m (aref buffer ind))))
                       (incf num-received))))
            (subscribe "ping"
                       'test_roslisp-msg:ComplexMessage
                       #'ping-callback)
            (subscribe "pang"
                       'test_roslisp-msg:ComplexMessage
                       #'pang-callback)))
        (sleep 3))
      (and ok-flag (> num-received 10)))
  t)

(deftest test-send-receive-simple
    ;; depends on talker node and listener node. Talker sends string msgs on /chatter, listener echoes them on /chatter_echo
    (let ((num-received 0))
      (with-ros-node ("roslisp-tester")
        (labels ((chatter-callback (m)
                   (incf num-received)))
          (subscribe "chatter_echo"
                     'std_msgs-msg:string
                     #'chatter-callback)
          (loop for i to 180
                do (if (> num-received 5)
                     (return)
                     (progn
                       (ros-info (test-roslisp) "waited ~a of ~a second for 5 messages over /chatter_echo. Got ~a" i 180 num-received)
                       (sleep 1))))))
      (> num-received 5))
    t)

(deftest test-broken-talker
    ;; depends on listener node with broken callback function which errors when we send string "error". This test checks the node remains responsive after that.
    (let ((num-received 0))
      (with-ros-node ("roslisp-tester")
        (labels ((chatter-callback (m)
                   (incf num-received)))
          (let ((pub (advertise "error_chatter" 'std_msgs-msg:string)))
            (subscribe "error_chatter_echo"
                       'std_msgs-msg:string
                       #'chatter-callback)
            (publish-msg pub :data "error")
            (sleep 1)
            (loop for i to 5 do
              (publish-msg pub :data (write-to-string i)))
            (loop for i to 180
                  do (if (> num-received 5)
                       (return)
                       (progn
                         (ros-info (test-roslisp) "waited ~a of ~a second for 5 messages over /error_chatter_echo. Got ~a" i 180 num-received)
                         (sleep 1)))))))
      (> num-received 5))
    t)


(create-suite "roslisp-topic-test")
